#!/bin/bash

# Copyright UiPath 2021
#
# =================
# LICENSE AGREEMENT
# -----------------
#   Use of paid UiPath products and services is subject to the licensing agreement
#   executed between you and UiPath. Unless otherwise indicated by UiPath, use of free
#   UiPath products is subject to the associated licensing agreement available here:
#   https://www.uipath.com/legal/trust-and-security/legal-terms (or successor website).
#   You must not use this file separately from the product it is a part of or is associated with.

NODE_ROLE=NODE_ROLE_PLACEHOLDER
BASE_BACKUP_DIR=BASE_BACKUP_DIR_PLACEHOLDER
NFS_MOUNT_DIR=NFS_MOUNT_DIR_PLACEHOLDER

function get_secret_key() {
  local key=$1
  local namespace config_name
  namespace="uipath-infra"
  config_name="service-cluster-configurations"

  value=$(kubectl get secret ${config_name} \
    -o jsonpath="{.data.""${key^^}""}" \
    -n ${namespace} --ignore-not-found 2>/dev/null) || true

  echo -n "$(base64 -d <<<"$value")"
}

function is_null_or_empty() {
  local value="$1"
  local result="false"
  if [[ -z "${value}" || "${value}" == " " || "${value}" == "null" ]]; then
    result="true"
  fi
  echo "${result}"
}

function get_docker_secret_key() {
  local key="$1"
  local value
  value=$(get_secret_key "infra.docker_registry.${key}")
  # also check the old location used to store these
  [[ -n "${value}" ]] \
    || value=$(kubectl -n uipath get secrets docker-registry-credentials -o jsonpath="{.data.${key}}" --ignore-not-found | base64 -d -w 0) || true

  echo "${value}"
}

function create_disaster_recovery_cluster_config() {
  local configFile="$1"
  local rke2_token fixed_rke2_address fqdn docker_username docker_password profile tmp
  echo "start creating config file ${configFile}"
  tmp=$(mktemp)
  # reset current file
  echo "{}" > "${configFile}"
  rke2_token=$(get_secret_key "rke_token")
  if [[ $(is_null_or_empty "${rke2_token}") == "true" ]]; then
    echo "rke2 token can't be empty as cluster rollback required valid rke2"
    exit 0
  fi

  fixed_rke2_address=$(get_secret_key "fixed_rke_address")
  if [[ $(is_null_or_empty "${fixed_rke2_address}") == "true" ]]; then
    echo "fixed_rke2_address can't be empty as cluster rollback required fixed_rke2_address"
    exit 0
  fi

  docker_username=$(get_docker_secret_key "username")
  docker_password=$(get_docker_secret_key "password")

  fqdn=$(get_secret_key "cluster_fqdn")
  if [[ $(is_null_or_empty "${fqdn}") == "true" ]]; then
    echo "fqdn can't be empty as cluster rollback required fqdn"
    exit 0
  fi

  profile=$(get_secret_key "profile")
  if [[ $(is_null_or_empty "${profile}") == "true" ]]; then
    echo "profile can't be empty as cluster rollback required this value"
    exit 0
  fi

  local use_cilium
  use_cilium=$(grep -c cilium /etc/rancher/rke2/config.yaml || true)
  jq --arg docker_username "$docker_username" '.infra.docker_registry.username = $docker_username' "${configFile}" > "${tmp}" && mv "${tmp}" "${configFile}"
  jq --arg docker_password "$docker_password" '.infra.docker_registry.password = $docker_password' "${configFile}" > "${tmp}" && mv "${tmp}" "${configFile}"
  jq --arg fqdn "$fqdn" '.fqdn = $fqdn' "${configFile}" > "${tmp}" && mv "${tmp}" "${configFile}"
  jq --arg address "$fixed_rke2_address" '.fixed_rke_address = $address' "${configFile}" > "${tmp}" && mv "${tmp}" "${configFile}"
  jq --arg token "$rke2_token" '.rke_token = $token' "${configFile}" > "${tmp}" && mv "${tmp}" "${configFile}"
  jq --arg profile "$profile" '.profile = $profile' "${configFile}" > "${tmp}" && mv "${tmp}" "${configFile}"
  jq --arg use_cilium "$use_cilium" '.use_cilium = $use_cilium' "${configFile}" > "${tmp}" && mv "${tmp}" "${configFile}"

  echo "config file ${configFile} created"
}

if [[ "${NODE_ROLE}" != "agent" ]]; then
  export KUBECONFIG="/etc/rancher/rke2/rke2.yaml"
  export PATH=${PATH}:/var/lib/rancher/rke2/bin

  if kubectl -n uipath-infra get secret uipath-backup ; then
    backup_enabled=$(kubectl -n uipath-infra get secret uipath-backup -o json| jq -r ".data.backup_enabled"|base64 -d)
    if [[ "${backup_enabled}" != "true" ]]; then
      echo "backup is not enabled."
      # backup is not enabled
      exit 0
    fi
  fi

  # copy latest 2 snapshots to backup
  mkdir -p ${BASE_BACKUP_DIR}/snapshots
  # shellcheck disable=SC2012
  ls -trl /var/lib/rancher/rke2/server/db/snapshots --ignore='*part' | tail -n -2| awk '{print $9}'| xargs -I {} cp --preserve=timestamps -r /var/lib/rancher/rke2/server/db/snapshots/{} "${BASE_BACKUP_DIR}/snapshots/{}"

  # remove old snapshots
  # shellcheck disable=SC2012
  ls -tl ${BASE_BACKUP_DIR}/snapshots/| tail -n +4| awk '{print $9}'| xargs -I {} rm -rf "${BASE_BACKUP_DIR}/snapshots/{}"

  # We need to copy rancher encryption config json because of rancher has restore issue with version v1.21.4+rke2r2
  # We should remove this logic once rancher version is upgraded
  # https://uipath.atlassian.net/browse/SERFAB-5256
  echo "Start copying encryption-config.json file on nfs server"
  mkdir -p ${NFS_MOUNT_DIR}/cred/
  cp -r /var/lib/rancher/rke2/server/cred/encryption-config.json "${NFS_MOUNT_DIR}/cred/."

  # cluster config json
  tmpClusterConfigFile="$(mktemp cluster_config.XXXXXX)"
  clusterConfigFile="${BASE_BACKUP_DIR}/cluster_config.json"
  cluster_config_file_diff=""
  create_disaster_recovery_cluster_config "${tmpClusterConfigFile}"
  
  if [[ -f "${clusterConfigFile}" ]]; then
    echo "cluster config file ${clusterConfigFile} already present. Start comparing file content"
    cluster_config_file_diff=$(diff "${clusterConfigFile}" "${tmpClusterConfigFile}")
  fi
  if [[ -n "${cluster_config_file_diff}" || ! -f "${clusterConfigFile}" ]]; then
    echo "Start updating config file on nfs server"
    cp "${tmpClusterConfigFile}" "${clusterConfigFile}"
    echo "Config file copied on nfs server"
  fi
  rm -rf "${tmpClusterConfigFile}"
fi