[CmdletBinding()]
    param(
        [ValidateScript( { if (-Not ($_ | Test-Path -PathType Container)) { throw "Parameter ( -BasePath ) should be directory containing all the files/inputs to be validated." } return $true })]
        [string] $BasePath = ".",

        [ValidateScript( { if (-Not ($_ | Test-Path -PathType Container)) { throw "Parameter ( -TempLocation ) should be directory containing all the files/inputs to be validated." } return $true })]
        [string] $TempLocation = ".",

        [ValidateScript( { if (-Not ($_ | Test-Path -PathType Container)) { throw "Parameter ( -SecretLocation ) should be directory containing all the files/inputs to be validated." } return $true })]
        [string] $SecretLocation = "/run/secrets",

        [Parameter(Mandatory=$true)]
        [string[]] $EnabledServices,

        [Parameter(Mandatory=$true)]
        [string] $EnableIntegratedAuth,

        [Parameter(Mandatory=$true)]
        [string] $CreateDB
    )

function Main {
    try {
        if($env:KRB5CCNAME) {
            touch $env:KRB5CCNAME
        }
        Write-Host "starting preinstall validations for service inputs"
        #  Check and add additional certificate to container trusted store. This is required when sql server not using globally trusted certificate authority
        Set-AdditionalCertificate
        # check if sql host and port is accessable
        if($env:SQL_SERVER_HOST) {
            $IsDbConnected = Test-SqlHostPortConnectivity -SqlServerHost $env:SQL_SERVER_HOST -SqlServerPort $env:SQL_SERVER_PORT -ServiceName "aaaaa" -TestName "Host_Reachability_Check"
        }

        if ($CreateDB -eq $true) {
            Test-MasterDbConnectivity
        } else {
            $validationJson = "$env:SF_BASEPATH/validation.json"
            if (Test-Path -PathType Leaf -Path "$validationJson") {
                $validationConfig =  Get-Content -Path "$validationJson" | ConvertFrom-Json
                foreach ($service in $EnabledServices) {
                    $config = $validationConfig.Where({$_.service -eq $service})
                    if ($config) {
                        Test-ServiceDbConnectivity -Config $config
                    }
                }
            } else {
                throw "Validation.json missing"
            }
        }
    } catch {
        Write-Error "`nFailed: $_"
        Exit 1
    }
}

function Set-AdditionalCertificate {
    $filePath = "$env:SF_SECRETLOCATION/additional_ca_cert"
    if (Test-Path -Path $filePath -PathType Leaf) {
        Copy-Item $filePath -Destination "/etc/pki/ca-trust/source/anchors/additional_ca_cert.crt"
        update-ca-trust
    }
}

function Set-KerberosTgt {
    [CmdletBinding()]
    param(
        [Parameter(Mandatory)]
        [PSCustomObject] $ServiceName
    )
    $serviceGroupName = switch ($ServiceName) {
        "automationhub" {"discoverygroup"; Break}
        "taskmining" {"discoverygroup"; Break}
        Default {
            $serviceName
        }
    }
    Get-KerberosTgt -ServiceName $serviceGroupName
}

function Test-MasterDbConnectivity {

    Set-KerberosTgt -ServiceName "master"
    # Check if master db is accessible using given credentails
    $IsDbConnected = Test-DotNetDBConnection -ConnectionStringParameter "sql_server_default_connection" -ServiceName "master" -TestName "Database_Connectivity_Check"
    if ($IsDbConnected -ne 0) {
        return
    }

    # Check is user has right permission to create test database
    $IsTestDbCreated = New-Database -SqlDefaultConnectionParameter "sql_server_default_connection" -ServiceName "master" -TestName "Create_Database_Test" -DbName "AutomationSuite_TestDB"
    if ($IsTestDbCreated -ne 0) {
        return
    }

    # Add table and push test data
    $IsTestDataCreated = New-TableAndPushTestData -SqlDefaultConnectionParameter "sql_server_testdb_connection" -TableName "AutomationSuite_TestTable" -ServiceName "master" -TestName "Create_And_Edit_Data_Test"

    if ($IsTestDataCreated -eq 0) {
        foreach ($service in $EnabledServices) {
            # If documentunderstanding is enabled, validate FullTextSearch feature
            if ("$service" -eq "documentunderstanding" ) {
                Test-DotNetDBConnection-FullTextSearchEnabled -ServiceName "master"  -ConnectionStringParameter "sql_server_testdb_connection" -TestName "Database_FullTextSearch_Check"
            } elseif ("$service" -eq "insights" ) {
                # If insights is enable test compatibility level
                Test-DotNetDBConnection-CompatibilityLevel -ServiceName "master"  -ConnectionStringParameter "sql_server_testdb_connection" -TestName "Database_CompatibilityLevel_Check"
            } elseif ("$service" -eq "automationhub" ) {
                # If insights is enable test compatibility level
                Test-DotNetDBConnection-CompatibilityLevel -ServiceName "master"  -ConnectionStringParameter "sql_server_testdb_connection" -TestName "Database_CompatibilityLevel_Check" -CompatibilityLevelCheckOnly $true
            }
        }
    }
    # Drop test dabase
    Remove-Database -SqlDefaultConnectionParameter "sql_server_default_connection" -DbName "AutomationSuite_TestDB"
}

function Test-ServiceDbConnectivity {
    [CmdletBinding()]
    param(
        [Parameter(Mandatory)]
        [PSCustomObject] $Config
    )
    Set-KerberosTgt -ServiceName $Config.service
    $connectionStringParameters = @($Config.connectionStringParameter)
    foreach ($connectionStringParameter in $connectionStringParameters) {
        if($connectionStringParameter) {
            if ("$service" -eq "documentunderstanding" ) {
                $IsDbConnected = Test-ODBCConnection -ServiceName $Config.service -ConnectionStringParameter $connectionStringParameter -TestName "Database_Connectivity_Check"
                if ($IsDbConnected -ne 0) {
                    continue
                }
                Test-ODBCDatabaseRoles -ServiceName $Config.service -ConnectionStringParameter $connectionStringParameter -TestName "Database_Role_Check"
                Test-ODBCDBConnection-FullTextSearchEnabled -ServiceName $Config.service  -ConnectionStringParameter $connectionStringParameter -TestName "Database_FullTextSearch_Check"
            } else {
                $IsDbConnected = Test-DotNetDBConnection -ServiceName $Config.service -ConnectionStringParameter $connectionStringParameter -TestName "Database_Connectivity_Check"
                if ($IsDbConnected -ne 0) {
                    continue
                }
                Test-DotNetDatabaseRoles -ServiceName $Config.service -ConnectionStringParameter $connectionStringParameter -TestName "Database_Role_Check"
                if ("$service" -eq "insights" ) {
                    Test-DotNetDBConnection-CompatibilityLevel -ServiceName $Config.service  -ConnectionStringParameter $connectionStringParameter -TestName "Database_CompatibilityLevel_Check"
                }
                elseif ("$service" -eq "automationhub" ) {
                    # If insights is enable test compatibility level
                    Test-DotNetDBConnection-CompatibilityLevel -ServiceName $Config.service  -ConnectionStringParameter $connectionStringParameter -TestName "Database_CompatibilityLevel_Check" -CompatibilityLevelCheckOnly $true
                }
            }
        }
    }
}

$env:SF_BASEPATH=(Resolve-Path $BasePath).path
$env:SF_TEMPLOCATION=(Resolve-Path $TempLocation).path
$env:SF_SECRETLOCATION=(Resolve-Path $SecretLocation).path
$env:VALIDATION_REPORT_FILE_LOCATION="$env:SF_TEMPLOCATION"

. "$env:SF_BASEPATH/Utils.ps1"
Main
