[CmdletBinding()]
    param(
        [ValidateScript( { if (-Not ($_ | Test-Path -PathType Leaf)) { throw "Tests file path parameter ( -TestsFilePath ) is not valid." } return $true })]
        [string] $TestsFilePath = "tests.json",
        
        [Parameter(Mandatory = $true)]
        $TestRailUserName,

        [Parameter(Mandatory = $true)]
        $TestRailPass,

        [Parameter(Mandatory = $true)]
        [string] $TestRunName,

        [Parameter(Mandatory = $true)]
        [string] $TestRunId,

        [Parameter(Mandatory = $true)]
        $TestRailProjectID,

        [Parameter(Mandatory = $true)]
        $TestRailSuiteID,

        [Parameter(Mandatory = $true)]
        [string] $InstallParams,

        [Parameter(Mandatory = $true)]
        [string] $TestRailTags,

        [Parameter(Mandatory = $false)]
        $TestRailCompanyDomain = "uipath.testrail.com",

        [Parameter(Mandatory = $false)]
        $ClusterType = "Online"
    )

$ErrorActionPreference = "Continue"

function Main {
    
    $testsJson = Read-TestsFile -TestsFilePath $testsFilePath
    $installParamsJson = Read-StringToDictJson -InputString $installParams
    $testTagsJson = Read-StringToListJson -InputString $testRailTags

    Set-AuthVariables
    $testResults = Invoke-AllTests -TestsJson $testsJson -InstallParams $installParamsJson  -TestTags $testTagsJson
    Write-Host "$(Get-Date) Results object type is: $($testResults.GetType())"
    Write-ResultsFile -TestResults $testResults -OutputPath "testrail-tests.xml"
}

function Read-StringToDictJson {

    [CmdletBinding()]
    param(
        [Parameter(Mandatory)]
        [String] $InputString
    )

    $jsonContent = @{}
    $InputString.split(",") | ForEach-Object {
        $Tag = $_
        $key,$value=$Tag.split("=")
        $jsonContent[$key]=$value
    }

    return $jsonContent
}

function Read-StringToListJson {

    [CmdletBinding()]
    param(
        [Parameter(Mandatory)]
        [String] $InputString
    )
    # remove whitespaces
    $InputString = $InputString.Replace(" ", "")
    $inputList = $InputString.split(",")
    return $inputList
}

function Read-TestsFile {
    [CmdletBinding()]
    param(
        [Parameter(Mandatory)]
        [ValidateScript( { Test-Path -Path $_ -PathType Leaf })]
        [ValidatePattern('\.json$')]
        [String] $TestsFilePath
    )

    if ( -Not (Test-Path $TestsFilePath -PathType Leaf)) {
        Write-Error "Input file $TestsFilePath not found"
    }
    $jsonContent = Get-Content -Path $TestsFilePath | ConvertFrom-Json -Depth 100
    
    return $jsonContent
}

function Invoke-AllTests {
    [CmdletBinding()]
    param(
        [Parameter(Mandatory)]
        [System.Object[]] $TestsJson,

        [Parameter(Mandatory)]
        [System.Object[]] $InstallParams,

        [Parameter(Mandatory)]
        [System.Object[]] $TestTags
    )

    $testResults = @{}
    $StopWatch = New-Object -TypeName System.Diagnostics.Stopwatch
    if ( $TestRunId -eq "0" ) {
        $tmp = New-TestRun
        $testRun = $tmp.id
    } else {
        $testRun = $TestRunId
    }
    $Env:ANSIBLE_STDOUT_CALLBACK = "yaml"
    $TestsJson | ForEach-Object {
        $crtTest = $_
        
        if (!$testResults[$crtTest.TestSuite]){
            $testResults.Add($crtTest.TestSuite, @{})
        }
        
        if ($TestTags -ne "" -and $TestTags -notcontains $crtTest.Tags) {
            Write-Host "$(Get-Date)==============Skipping up testcase for $($crtTest.TestName) with tags $TestTags=============="
            return;
        }


        $ClusterTypeLower = $ClusterType.ToLower()
        if ( $crtTest.ClusterType -notcontains $ClusterType) {
            Write-Host "$(Get-Date)==============Skipping up testcase for $($crtTest.TestName) with clusterType $ClusterType=============="
            return;
        }

        if ( Test-Path -Path "./Tests/$ClusterTypeLower/test_results.json" -PathType Leaf ){
            Remove-Item -Path "./Tests/$ClusterTypeLower/test_results.json" -Force
        }

        # Skip Tests based on tags
        if ($crtTest.InstallType -eq "DisasterRecovery" -and $InstallParams.disasterRecovery -ne $true){
            Write-Host "==============Skip tests for $($crtTest.TestName) as selected setup is non-DR =============="
            return;
        }

        if ($crtTest.TestSuite -eq "SingleNode" -and $InstallParams.serverNodeCount -gt 1){
            Write-Host "==============Skip tests for $($crtTest.TestName) as selected setup is HA =============="
            return;
        }

        if ($crtTest.TestSuite -eq "MultiNode" -and $InstallParams.serverNodeCount -lt 3){
            Write-Host "==============Skip tests for $($crtTest.TestName) as selected setup is non-HA =============="
            return;
        }


        if ($crtTest.RunPrerequisites) {
            Write-Host "$(Get-Date)==============Setting up prereqs for $($crtTest.TestName)=============="
            $inputJsonContent = Get-Content -Path $ClusterTypeLower/all_input.json | ConvertFrom-Json

            if ($crtTest.Tags -eq "Alerts"){
                $inputJsonContent.objectstore.pvc_size = "10Gi";
            }
            
            if ($crtTest.TestSuite -eq "SingleNode"){
                $inputJsonContent.profile = "default";
                $inputJsonContent | ConvertTo-Json | Set-Content -Path "$ClusterTypeLower/all_input.json"
            } else{
                $inputJsonContent.profile = "ha";
                $inputJsonContent | ConvertTo-Json | Set-Content -Path "$ClusterTypeLower/all_input.json"
            }

            Invoke-Expression -Command "ansible-playbook $ClusterTypeLower/prereq.ansible.yml -i $ClusterTypeLower/inventory.ini -vv" -ErrorAction Continue 2>&1 | Write-Host
            Write-Host "$(Get-Date)==============Finished setting up prereqs for $($crtTest.TestName)=============="
        } else {
            Write-Host "$(Get-Date)==============Skipping prereqs for $($crtTest.TestName)=============="
        }

        if ($crtTest.PerformInstall) {
            $inputJsonContent = Get-Content -Path $ClusterTypeLower/all_input.json | ConvertFrom-Json
            Invoke-Expression -Command "ansible-playbook $ClusterTypeLower/uninstall.ansible.yml -i $ClusterTypeLower/inventory.ini -vv" -ErrorAction Continue 2>&1 | Write-Host
            if ($inputJsonContent.profile -eq "ha")  {
                Invoke-Expression -Command "ansible-playbook $ClusterTypeLower/multi-node-install.ansible.yml -i $ClusterTypeLower/inventory.ini -vv" -ErrorAction Continue 2>&1 | Write-Host
            }
            else {
                Invoke-Expression -Command "ansible-playbook $ClusterTypeLower/single-node-install.ansible.yml -i $ClusterTypeLower/inventory.ini -vv" -ErrorAction Continue 2>&1 | Write-Host
            }
            Write-Host "$(Get-Date)==============Setting up installation for $($crtTest.TestName)=============="
            Write-Host "$(Get-Date)==============Finished setting up prereqs for $($crtTest.TestName)=============="
        } else {
            Write-Host "$(Get-Date)==============Skipping installation for $($crtTest.TestName)=============="
        }

        $StopWatch.Start()
        Write-Host "$(Get-Date)==============Running test: $($crtTest.TestName)=============="
        Invoke-Expression -Command "ansible-playbook ./Tests/$($_.Entrypoint) -i $ClusterTypeLower/inventory.ini -vv" -ErrorAction Continue | Tee-Object -FilePath output | Write-Host
        $StopWatch.Stop()

        if ( Test-Path -Path "./Tests/test_results.json" -PathType Leaf ){
            Write-Host "$(Get-Date)==============Successfully executed test: $($crtTest.TestName)=============="
            $crtSuiteResults = Get-Content -Path "./Tests/test_results.json" | ConvertFrom-Json -Depth 100
            $crtSuiteResults | ForEach-Object {
                $crtResult = $_
                if ( $crtResult.result ){
                    Write-Host "$(Get-Date)==============Successfully validated test: $($crtResult.test)=============="
                    $testResults[$crtTest.TestSuite].Add($crtResult.test, @{})
                    $testResults[$crtTest.TestSuite][$crtResult.test].Add("Duration", $StopWatch.Elapsed.TotalSeconds)
                    $testResults[$crtTest.TestSuite][$crtResult.test].Add("Message", "Success")

                    Add-TestResult -TestRunId $testRun -TestCaseId $crtResult.test -TestRunResult 1 -ElapsedTime $StopWatch.Elapsed.TotalSeconds
                } else {
                    Write-Error "$(Get-Date)==============Error while validating test: $($crtResult.test)=============="
                    $testResults[$crtTest.TestSuite].Add($crtResult.test, @{})
                    $testResults[$crtTest.TestSuite][$crtResult.test].Add("Duration", $StopWatch.Elapsed.TotalSeconds)
                    $testResults[$crtTest.TestSuite][$crtResult.test].Add("Message", "Error")

                    Add-TestResult -TestRunId $testRun -TestCaseId $crtResult.test -TestRunResult 5 -ElapsedTime $StopWatch.Elapsed.TotalSeconds -ErrorMessage $fullError
                }
            }
        }
        if ($crtTest.PerformUninstall) {
            Write-Host "$(Get-Date)==============Uninstalling after test $($crtTest.TestName)=============="
            Invoke-Expression -Command "ansible-playbook $ClusterTypeLower/uninstall.ansible.yml -i $ClusterTypeLower/inventory.ini -vv" -ErrorAction Continue 2>&1 | Write-Host
            Write-Host "$(Get-Date)==============Uninstalling finished for test $($crtTest.TestName)=============="
        } else {
            Write-Host "$(Get-Date)==============Skipping uninstall after test $($crtTest.TestName)=============="
        }
        $StopWatch.Reset()
    }
    return $testResults
}

function Write-ResultsFile {
    [CmdletBinding()]
    param(
        [Parameter(Mandatory)]
        [ValidatePattern('\.xml$')]
        [String] $OutputPath,

        [Parameter(Mandatory)]
        [Hashtable] $TestResults
    )

    # Set The Formatting
    [xml.XMLDocument]$xmlOutput = New-Object Xml.XMLDocument

    # Start the Root Element
    [System.XML.XMLElement]$rootElement = $xmlOutput.CreateElement("testsuites")
    $xmlOutput.AppendChild($rootElement)

    foreach ($crtSuite in $TestResults.Keys) {

        [XML.XMLElement]$crtNode = $rootElement.AppendChild($xmlOutput.CreateElement("testsuite"))
        $totalTime = 0
        $totalTests = 0
        $totalFailures = 0
        $crtNode.SetAttribute("name", $crtSuite)
        
        foreach ($crtTest in $TestResults[$crtSuite].Keys){
            $totalTests++
            $totalTime += $TestResults[$crtSuite][$crtTest]["Duration"]
            [System.XML.XMLElement]$crtTestNode = $xmlOutput.CreateElement("testcase")
            $crtNode.AppendChild($crtTestNode)
            $crtTestNode.SetAttribute("name", $crtTest)
            $crtTestNode.SetAttribute("time", $TestResults[$crtSuite][$crtTest]["Duration"])
            if ($TestResults[$crtSuite][$crtTest]["Message"] -ne "Success"){
                [System.XML.XMLElement]$failureNode = $xmlOutput.CreateElement("failure")
                $crtTestNode.AppendChild($failureNode)
                $failureNode.SetAttribute("message", $TestResults[$crtSuite][$crtTest]["Message"])
                $totalFailures++
            }
        }
        $crtNode.SetAttribute("time", $totalTime)
        $crtNode.SetAttribute("errors", $totalFailures)
        $crtNode.SetAttribute("tests", $totalTests)
    }
    $xmlOutput.Save($OutputPath)
}

function New-TestRun {

    $testRunDetails = @{
        suite_id=$TestRailSuiteID
        name=$TestRunName
        include_all=$true
    } | ConvertTo-Json

    $uri = New-Object System.UriBuilder -argumentlist 'https', $TestRailCompanyDomain, 443, '/index.php', "?/api/v2/add_run/$TestRailProjectID"
    try {
        $testRun = Invoke-RestMethod -Method POST -Uri $uri.Uri.AbsoluteUri -Headers $headers -Body $testRunDetails -UseBasicParsing -TimeoutSec 120
    }
    catch {
        $StatusCode=$_.Exception.Response.StatusCode.value__
        if ( $StatusCode -ne 200 ) {
            Write-Error "Failed to create test run with status code $StatusCode."
        }
    }

    return $testRun
}

function Add-TestResult {
    param(
        [Parameter(Mandatory = $true)]
        [string]$TestRunId,

        [Parameter(Mandatory = $true)]
        $TestCaseId,

        # Test Run result codes
        # 1	Passed
        # 2	Blocked
        # 3	Untested (not allowed when adding a result)
        # 4	Retest
        # 5	Failed
        [Parameter(Mandatory = $true)]
        [int]$TestRunResult,

        [Parameter(Mandatory = $true)]
        [int]$ElapsedTime,

        [Parameter(Mandatory = $false)]
        [string]$ErrorMessage = ""
    )
    
    if ($ErrorMessage.Length -gt 240) {
        $ErrorMessage = $ErrorMessage.Substring($ErrorMessage.Length - 240);
    }

    $testCaseRunDetails = @{
        status_id=$TestRunResult
        version=$TestRunName
        elapsed=$ElapsedTime
        defects=$ErrorMessage
    } | ConvertTo-Json

    foreach ($id in $TestCaseId) {
        Write-Host "This is the ID: $id, testRun ID is: $TestRunId"
        $uri = New-Object System.UriBuilder -argumentlist 'https', $TestRailCompanyDomain, 443, '/index.php', "?/api/v2/add_result_for_case/$TestRunId/$id"
        try {
            Invoke-RestMethod -Method POST -Uri $uri.Uri.AbsoluteUri -Headers $headers -Body $testCaseRunDetails -UseBasicParsing -TimeoutSec 120 | Write-Host
        }
        catch {
            $StatusCode=$_.Exception.Response.StatusCode.value__
            if ( $StatusCode -ne 200 ) {
                Write-Error "Failed to update testrail for testid $id with status code $StatusCode."
            }
        }
    }
}

function Set-AuthVariables {
    
    $Script:base64AuthInfo = [Convert]::ToBase64String([Text.Encoding]::ASCII.GetBytes(("{0}:{1}" -f $TestRailUserName, $TestRailPass)))
    $Script:headers = @{
        Authorization=("Basic {0}" -f $base64AuthInfo)
        custom_automation_type="SF-pipeline"
        "Content-Type"="application/json"
    }
}

Main
Exit 0
