#!/bin/bash

# Copyright UiPath 2021
#
# =================
# LICENSE AGREEMENT
# -----------------
#   Use of paid UiPath products and services is subject to the licensing agreement
#   executed between you and UiPath. Unless otherwise indicated by UiPath, use of free
#   UiPath products is subject to the associated licensing agreement available here:
#   https://www.uipath.com/legal/trust-and-security/legal-terms (or successor website).
#   You must not use this file separately from the product it is a part of or is associated with.

#GLOBALS:
SCRIPT_DIR=$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)
if [[ -z "${SCRIPT_DIR}" ]]; then
  error "Could not determine script path"
fi
MODULES_DIR=$(cd "${SCRIPT_DIR}/../../Modules" && pwd)
if [[ -z "${MODULES_DIR}" ]]; then
  error "Could not determine modules path"
fi
#shellcheck disable=SC1090,SC1091
source "${MODULES_DIR}/utils.sh"
#shellcheck disable=SC1090,SC1091
source "${MODULES_DIR}/upgrade_utils.sh"
#shellcheck disable=SC1090,SC1091
source "${MODULES_DIR}/argocd_utils.sh"

####
# Sets current rook-version and ceph-version from all pods
####
#shellcheck disable=SC2034
function set_rook_ceph_version() {
    local rook_cluster_namespace=rook-ceph
    current_rook_version=$(kubectl -n $rook_cluster_namespace get deployment -l rook_cluster=$rook_cluster_namespace -o jsonpath='{range .items[*]}{"rook-version="}{.metadata.labels.rook-version}{"\n"}{end}' | sort | uniq)
    # current_ceph_version=$(kubectl -n $rook_cluster_namespace get deployment -l rook_cluster=$rook_cluster_namespace -o jsonpath='{range .items[*]}{"ceph-version="}{.metadata.labels.ceph-version}{"\n"}{end}' | sort | uniq)
}

OSD_ALLOWED_TO_CLEANUP=$(kubectl -n rook-ceph get deploy -l app=rook-ceph-osd  -o json | jq -c '[.items[] | .metadata.name]')
if [[ -z "${OSD_ALLOWED_TO_CLEANUP}" || $OSD_ALLOWED_TO_CLEANUP == "[]" ]]; then
    warn "Failure getting OSD deployment. Setting it to default"
    OSD_ALLOWED_TO_CLEANUP='["rook-ceph-osd-0","rook-ceph-osd-1","rook-ceph-osd-2"]'
fi

function cleanup_crashing_osd() {
    local restart_operator="false"
    local min_required_healthy_osd=1
    local in_osd
    local up_osd
    local healthy_osd_pod_count
    local crashed_osd_deploy
    local crashed_pvc_name

    if ! kubectl -n rook-ceph exec deploy/rook-ceph-tools -- ceph osd pool ls detail  | grep 'rook-ceph.rgw.buckets.data' | grep -q 'replicated'; then
        min_required_healthy_osd=2
    fi
    in_osd=$(kubectl -n rook-ceph exec deploy/rook-ceph-tools -- ceph status   -f json  | jq -r '.osdmap.num_in_osds')
    up_osd=$(kubectl -n rook-ceph exec deploy/rook-ceph-tools -- ceph status   -f json  | jq -r '.osdmap.num_up_osds')
    healthy_osd_pod_count=$(kubectl -n rook-ceph get pod -l app=rook-ceph-osd | grep 'Running' | grep -c -P '([0-9])/\1')
    if ! [[ $in_osd -ge $min_required_healthy_osd && $up_osd -ge $min_required_healthy_osd && $healthy_osd_pod_count -ge $min_required_healthy_osd ]]; then
        return
    fi
    for crashed_osd_deploy in $(kubectl -n rook-ceph get pod -l app=rook-ceph-osd  | grep 'CrashLoopBackOff' | cut -d'-' -f'1-4') ; do
        if kubectl -n rook-ceph logs "deployment/${crashed_osd_deploy}" | grep -q '/crash/'; then
            echo "Found crashing OSD deployment: '${crashed_osd_deploy}'"
            if ! jq -n  -c --argjson allowedOSDs "${OSD_ALLOWED_TO_CLEANUP}"  --arg osdDeploy "${crashed_osd_deploy}" '$allowedOSDs | index($osdDeploy)' | grep -q -E '^[0-9]+$'; then
                error "Won't try recovery of OSD deployment: '${crashed_osd_deploy}' as it is not part of allowed list: '${OSD_ALLOWED_TO_CLEANUP}'. Please work with Uipath support to complete the upgrade"
            fi
            crashed_pvc_name=$(kubectl -n rook-ceph get deployment "${crashed_osd_deploy}" -o json | jq -r '.metadata.labels["ceph.rook.io/pvc"]')
            info "Removing crashing OSD deployment: '${crashed_osd_deploy}' and PVC: '${crashed_pvc_name}'"
            timeout 60  kubectl -n rook-ceph delete deployment "${crashed_osd_deploy}" || kubectl -n rook-ceph delete deployment "${crashed_osd_deploy}" --force --grace-period=0
            timeout 100 kubectl -n rook-ceph delete pvc "${crashed_pvc_name}" || kubectl -n rook-ceph delete pvc "${crashed_pvc_name}" --force --grace-period=0
            OSD_ALLOWED_TO_CLEANUP=$(jq -n  -c --argjson allowedOSDs "${OSD_ALLOWED_TO_CLEANUP}" --arg osdDeploy "${crashed_osd_deploy}"  '$allowedOSDs | del(.[] | select(. == $osdDeploy))')
            restart_operator="true"
        fi
    done
    if [[ $restart_operator == "true" ]]; then
        kubectl -n rook-ceph rollout restart deployment/rook-ceph-operator
    fi
    return 0
}

function validate_rook_ceph_version() {
    local return_code=0
    local rook_cluster_namespace=rook-ceph
    local required_rook_version="sf-rook-ceph-fork:v1.8.9-4"
    local required_ceph_version="ceph/ceph:v15.2.10"

    kubectl -n "${rook_cluster_namespace}" get deploy rook-ceph-operator -o json | jq -r '.spec.template.spec.containers[0].image' | grep -q -v -F "${required_rook_version}" && return_code=1
    kubectl -n "${rook_cluster_namespace}" get deploy -l rook_cluster="${rook_cluster_namespace}" -o json | jq -r '.items[].spec.template.spec.containers[0].image' | grep -q -v -F "${required_ceph_version}" && return_code=1
    return "${return_code}"
}

function is_ceph_osd_on_pv_exists(){
  local rook_ceph_namespace="rook-ceph"
  local exit_code=1

  kubectl -n "${rook_ceph_namespace}" get deployment  -l app=rook-ceph-osd,portable=true -o name | grep -q 'deployment' && exit_code=0
  return "${exit_code}"
}


function wait_until_required_osd_on_raw_disk_healthy(){
    local rook_ceph_namespace="rook-ceph"
    local master_node_count=0
    local ceph_osd_on_raw_disk_ids
    local ceph_osd_on_raw_disk_healthy_ids
    local ceph_osd_on_raw_disk_healthy_count=0
    local start_time
    local current_time
    local timeout=900
    local diff

    start_time=$(date +"%s")
    while [[ $master_node_count -eq 0 ]] || [[ $ceph_osd_on_raw_disk_healthy_count -lt $master_node_count ]]; do
        current_time=$(date +"%s")
        diff=$((current_time-start_time))
        [[ $diff -gt $timeout ]] && error "Timeout waiting for ceph osd on raw device to become healthy. Required('${master_node_count}'), Current('${ceph_osd_on_raw_disk_healthy_count}')"
        [[ $master_node_count -ne 0 ]] && sleep 5
        master_node_count=$(kubectl get node -l node-role.kubernetes.io/master=true -o name  | grep -c 'node/')
        ceph_osd_on_raw_disk_ids=$(kubectl -n "${rook_ceph_namespace}" get deployment  -l app=rook-ceph-osd,portable=false -o json | jq -c '[.items[] | .metadata.labels["ceph-osd-id"] | tonumber]')
        ceph_osd_on_raw_disk_healthy_ids=$(kubectl -n "${rook_ceph_namespace}" exec deploy/rook-ceph-tools -- ceph osd dump -f json | jq -c --argjson rawOsdIds "${ceph_osd_on_raw_disk_ids}"  '[.osds[] | select((.osd as $osd | $rawOsdIds | index($osd)) and .in==1 and .up==1).osd | tostring]')
        ceph_osd_on_raw_disk_healthy_count=$(echo "${ceph_osd_on_raw_disk_healthy_ids}" | jq 'length')
    done

}


function is_ceph_osd_on_raw_disk_exists(){
  local rook_ceph_namespace="rook-ceph"
  local exit_code=1

  kubectl -n "${rook_ceph_namespace}" get deployment  -l app=rook-ceph-osd,portable=false -o name | grep -q 'deployment' && exit_code=0
  return "${exit_code}"
}


function get_osd_on_raw_info() {
    local rook_ceph_namespace="rook-ceph"
    RAW_OSD_ID=$(kubectl -n "${rook_ceph_namespace}" exec deploy/rook-ceph-tools -- ceph osd dump -f json | jq -c --argjson osd_to_remove "${PV_OSD_ID}"  '[.osds[] | select((.osd as $osd | $osd_to_remove | index($osd) | not) and .in==1 and .up==1).osd | tostring]')
    RAW_OSD_COUNT_BY_ZONE=$(kubectl -n "${rook_ceph_namespace}" get deploy -o json | jq --argjson osd_id "${RAW_OSD_ID}" '[.items[] | select(.metadata.labels["ceph-osd-id"] as $osd | $osd_id | index($osd)) | {"zone" : .metadata.labels["topology-location-zone"], "osd_id": .metadata.labels["ceph-osd-id"] }] | group_by(.zone)[] |  {(.[0].zone): [.[] | .osd_id]} | to_entries[] | { (.key) : (.value | length)}')
    RAW_OSD_COUNT_BY_HOST=$(kubectl -n "${rook_ceph_namespace}" get deploy -o json | jq --argjson osd_id "${RAW_OSD_ID}" '[.items[] | select(.metadata.labels["ceph-osd-id"] as $osd | $osd_id | index($osd)) | {"zone" : .metadata.labels["topology-location-host"], "osd_id": .metadata.labels["ceph-osd-id"] }] | group_by(.zone)[] |  {(.[0].zone): [.[] | .osd_id]} | to_entries[] | { (.key) : (.value | length)}')
    RAW_OSD_UNIQ_HOST_COUNT=$(echo "$RAW_OSD_COUNT_BY_HOST" | jq -s 'keys | length')

    [[ "${RAW_OSD_ID}" == '[]' ]] && error "[ERROR] No Healthy OSD will present in cluster if PV OSDs are removed"
    [[ -z "${RAW_OSD_COUNT_BY_ZONE}" ]] && error "[ERROR] EMPTY 'RAW_OSD_COUNT_BY_ZONE'"
    [[ -z "${RAW_OSD_COUNT_BY_HOST}" ]] && error "[ERROR] EMPTY 'RAW_OSD_COUNT_BY_HOST'"
}

function get_osd_on_pv_info() {
    local rook_ceph_namespace="rook-ceph"
    PV_OSD_ID=$(kubectl -n "${rook_ceph_namespace}" get deploy -l app=rook-ceph-osd,portable=true -o json | jq -c '[.items[] | .metadata.labels["ceph-osd-id"] | tonumber]')
    PV_OSD_COUNT_BY_ZONE=$(kubectl -n "${rook_ceph_namespace}" get deploy -l app=rook-ceph-osd,portable=true -o json | jq -c '[.items[] | {"zone" : ( try( .metadata.labels["topology-location-zone"]) // "def_zone"), "osd_id": .metadata.labels["ceph-osd-id"] }] | group_by(.zone)[] |  {(.[0].zone): [.[] | .osd_id]} | to_entries[] | { (.key) : (.value | length)}')
    PV_OSD_COUNT_BY_HOST=$(kubectl -n "${rook_ceph_namespace}" get deploy -l app=rook-ceph-osd,portable=true -o json | jq -c '[.items[] | {"zone" : .metadata.labels["topology-location-host"], "osd_id": .metadata.labels["ceph-osd-id"] }] | group_by(.zone)[] |  {(.[0].zone): [.[] | .osd_id]} | to_entries[] | { (.key) : (.value | length)}')
    PV_OSD_UNIQ_HOST_COUNT=$(echo "$PV_OSD_COUNT_BY_HOST" | jq -s 'keys | length')

    [[ "${PV_OSD_ID}" == '[]' ]] && error "[ERROR] OSD backed by PV not found in the cluster"
    [[ -z "${PV_OSD_COUNT_BY_ZONE}" ]] && error "[ERROR] EMPTY 'PV_OSD_COUNT_BY_ZONE'"
    [[ -z "${PV_OSD_UNIQ_HOST_COUNT}" ]] && error "[ERROR] EMPTY 'PV_OSD_UNIQ_HOST_COUNT'"
}

function is_safe_to_remove_pv_osd(){
    local pv_osd_id
    local rook_ceph_namespace="rook-ceph"
    local start_time
    local current_time
    local timeout=600
    local diff

    start_time=$(date +"%s")
    for pv_osd_id in $(jq -n --argjson pv_osd_id "${PV_OSD_ID}" '$pv_osd_id[]'); do
        echo "Verifying if PV osd.${pv_osd_id} is safe to destroy"
        while ! kubectl -n "${rook_ceph_namespace}" exec   deploy/rook-ceph-tools -- ceph osd safe-to-destroy "osd.${pv_osd_id}" ; do
            current_time=$(date +"%s")
            diff=$((current_time-start_time))
            [[ $diff -gt $timeout ]] && error "Timeout waiting to ensure PV OSDs are safe to remove"
            sleep 10
        done
      echo "PV osd.${pv_osd_id} is safe to destroy"
    done
}


function remove_osd_on_pv() {
    local pv_osd_id
    local rook_ceph_namespace="rook-ceph"
    local operator_image
    local osd_pv_deployment
    local try_pv_osd_removal_additionally="false"

    operator_image=$(kubectl -n "${rook_ceph_namespace}" get deploy/rook-ceph-operator -o jsonpath='{..image}')
    for pv_osd_id in $(jq -n --argjson pv_osd_id "${PV_OSD_ID}" '$pv_osd_id[]'); do
        osd_pv_deployment=$(kubectl -n "${rook_ceph_namespace}" get deploy -l "app=rook-ceph-osd,ceph-osd-id=${pv_osd_id}" -o name)
        [[ -n "${osd_pv_deployment}" ]] && kubectl -n "${rook_ceph_namespace}" scale --replicas=0 "${osd_pv_deployment}"
        while kubectl -n "${rook_ceph_namespace}" wait pod --for=delete  -l "app=rook-ceph-osd,portable=true,ceph-osd-id=${pv_osd_id}" --timeout=10s 2>&1 | grep --color=never 'timed out waiting for'; do sleep 5 ; done
        \cp "$MODULES_DIR"/../Fabric_Installer/k8s_manifests/rook-ceph-osd-purge.yaml "$MODULES_DIR"/../Fabric_Installer/k8s_manifests/rook-ceph-osd-purge-tmp.yaml
        sed -i "s;__IMAGE_PLACEHOLDER__;${operator_image};g" "$MODULES_DIR"/../Fabric_Installer/k8s_manifests/rook-ceph-osd-purge-tmp.yaml
        sed -i "s;__OSD_ID__;${pv_osd_id};g" "$MODULES_DIR"/../Fabric_Installer/k8s_manifests/rook-ceph-osd-purge-tmp.yaml
        kubectl -n "${rook_ceph_namespace}" delete job -l "app=rook-ceph-purge-osd"
        kubectl apply -f "$MODULES_DIR"/../Fabric_Installer/k8s_manifests/rook-ceph-osd-purge-tmp.yaml
        kubectl -n "${rook_ceph_namespace}" wait --for=condition=Complete job/rook-ceph-purge-osd --timeout=300s || error "Timeout waiting for osd purge job to complete for 'OSD.${pv_osd_id}'"
        if ! kubectl -n "${rook_ceph_namespace}" logs job/rook-ceph-purge-osd  | grep -q 'completed removal of OSD'; then
            warn "Unable to find required string in logs of osd purge job. Please verify manually if osd.${pv_osd_id} and its resources are removed correctly"
            try_pv_osd_removal_additionally="true"
            kubectl -n "${rook_ceph_namespace}" exec deploy/rook-ceph-tools -- ceph osd purge "osd.${pv_osd_id}" --force --yes-i-really-mean-it || true
            kubectl -n "${rook_ceph_namespace}" delete deployment -l "app=rook-ceph-osd,ceph-osd-id=${pv_osd_id}" || true
        fi
    done
    if [[ $try_pv_osd_removal_additionally == "true" ]]; then
        kubectl -n "${rook_ceph_namespace}" delete deployment -l app=rook-ceph-osd,portable=true || true
        sleep 30
        kubectl -n "${rook_ceph_namespace}" delete persistentvolumeclaim -l ceph.rook.io/DeviceSet=set1 || true
    fi
}

function validate_remaining_ceph_cluster_capacity() {
    get_osd_on_pv_info
    get_osd_on_raw_info
    if jq -r -n --argjson pv_osd "${PV_OSD_COUNT_BY_ZONE}" --argjson remaining_osd "${RAW_OSD_COUNT_BY_ZONE}" '$pv_osd | to_entries[] | if (.key != "def_zone" and ($remaining_osd[.key] == null or $remaining_osd[.key] < .value)) then "validation_failed" else "" end' | grep -q 'validation_failed';then
        error "[Validation failed] Not enough replacement OSD present in the cluster for topology: zone" 
    fi
    if [[ $RAW_OSD_UNIQ_HOST_COUNT -lt $PV_OSD_UNIQ_HOST_COUNT ]]; then
        error "[Validation failed] Not enough replacement OSD present in the cluster for topology: host"
    fi
}

function mark_osd_on_pv_out(){
    local pv_osd_id
    local rook_ceph_namespace="rook-ceph"
    for pv_osd_id in $(jq -n --argjson pv_osd_id "${PV_OSD_ID}" '$pv_osd_id[]'); do
      kubectl -n "${rook_ceph_namespace}" exec deploy/rook-ceph-tools -- ceph osd out "osd.${pv_osd_id}"
    done
}

function remove_affinity_from_pv_backed_osds() {
  local rook_ceph_namespace="rook-ceph"
  local pvOsd

  handle_device_health_metrics_pool_config
  speed_up_ceph_recovery || true
  for pvOsd in $(kubectl -n "${rook_ceph_namespace}" get deployment -l app=rook-ceph-osd,portable=true -o name); do
    # Skip affinity removal if empty
    kubectl -n "${rook_ceph_namespace}" get "${pvOsd}" -o json | jq '.spec.template.spec.affinity=={}' | grep -q 'true' && continue
    kubectl -n "${rook_ceph_namespace}" patch "${pvOsd}"  -p='[{"op":"replace", "path": "/spec/template/spec/affinity", "value": {}}]' --type=json
    info "Removed affinity config from Ceph OSD: '${pvOsd}' running on PV"
    kubectl -n "${rook_ceph_namespace}" rollout status "${pvOsd}" --timeout=7m || true
    speed_up_ceph_recovery || true
  done
}

function migrate_pgs_from_pv_to_raw_osd_if_possible(){
    local timeout_seconds=600
    local current_time
    local start_time
    local diff

    if is_ceph_raw_disk_exists; then
        wait_until_ceph_cluster_using_raw_device_config
        remove_affinity_from_pv_backed_osds
        if ! is_ceph_osd_on_raw_disk_exists; then
            kubectl -n rook-ceph rollout restart deployment/rook-ceph-operator
            start_time=$(date +"%s")
            while ! is_ceph_osd_on_raw_disk_exists; do
                current_time=$(date +"%s")
                diff=$((current_time-start_time))
                [[ $diff -gt timeout_seconds ]] && error "Timeout waiting for osd to consume on raw device"
            done
        fi
    fi
    # Continue if osd on PV and Raw both are running in cluster
    if ! (is_ceph_osd_on_raw_disk_exists && is_ceph_osd_on_pv_exists); then
        return
    fi
    wait_until_required_osd_on_raw_disk_healthy
    validate_remaining_ceph_cluster_capacity
    mark_osd_on_pv_out
    wait_until_all_mon_updated
    force_upgrade_down_osd_on_pv
    wait_until_pg_drained
    is_safe_to_remove_pv_osd
    remove_osd_on_pv
}

function wait_until_all_mon_updated() {
    local image
    local timeout_seconds=1200
    local current_time
    local start_time
    local diff

    info "Checking if all monitors are upgraded"
    image=$(kubectl -n rook-ceph get cephcluster rook-ceph -o json | jq -r '.spec.cephVersion.image')
    start_time=$(date +"%s")
    while kubectl -n rook-ceph get deploy -l app=rook-ceph-mon -o wide --no-headers  | grep -q -v "${image}"; do
        current_time=$(date +"%s")
        diff=$((current_time-start_time))
        [[ $diff -gt $timeout_seconds ]] && error "Timeout waiting for all monitors to use image: '${image}'"
        sleep 10
    done
    info "All monitors are upgraded to '${image}'"
}

function wait_until_ceph_cluster_using_raw_device_config() {
    local timeout_seconds=600
    local current_time
    local start_time
    local diff
    local exit_code=1

    start_time=$(date +"%s")
    while ! kubectl -n rook-ceph get cephcluster rook-ceph -o json 2> /dev/null | jq '.spec.storage.devicePathFilter!=null' | grep -q 'true'; do
        current_time=$(date +"%s")
        diff=$((current_time-start_time))
        [[ $diff -gt timeout_seconds ]] && error "Timeout waiting for ceph cluster to use raw device config"
    done
}


function force_upgrade_down_osd_on_pv() {
    local image
    local old_osd_on_pv_ids
    local down_old_osd_on_pv_ids
    local osdDeploy

    image=$(kubectl -n rook-ceph get cephcluster rook-ceph -o json | jq -r '.spec.cephVersion.image')
    old_osd_on_pv_ids=$(kubectl -n rook-ceph get deploy -l app=rook-ceph-osd,portable=true -o json | jq -c  --arg image "${image}" '[.items[] | select(.spec.template.spec.containers[0].image != $image) | (.metadata.labels["ceph-osd-id"] | tonumber)]' )
    down_old_osd_on_pv_ids=$(kubectl -n rook-ceph exec deploy/rook-ceph-tools -- ceph osd dump -f json | jq -c --argjson oldOsdOnPvIds "${old_osd_on_pv_ids}"  '[.osds[] | select((.osd as $osd | $oldOsdOnPvIds | index($osd)) and .up!=1).osd]')
    for osd_id in $(jq -n --argjson downOldOsdOnPvIds "${down_old_osd_on_pv_ids}"  '$downOldOsdOnPvIds | .[]'); do
        osdDeploy=$(kubectl -n rook-ceph get deploy -l app=rook-ceph-osd,ceph-osd-id="${osd_id}" -o name)
        kubectl -n rook-ceph set image "${osdDeploy}" osd="${image}"
        kubectl -n rook-ceph rollout status "${osdDeploy}" --timeout=5m || true
    done
    speed_up_ceph_recovery || true
}

function handle_pgs_stuck_in_drain() {
    local stuck_osd_id="$1"
    local rook_ceph_namespace="rook-ceph"
    local pg_id
    local new_primary_osd

    for pg_id in $(kubectl -n "${rook_ceph_namespace}" exec deploy/rook-ceph-tools -- ceph pg ls-by-osd "osd.${stuck_osd_id}" --format json | jq -r '.pg_stats[].pgid'); do
        new_primary_osd=$(kubectl -n "${rook_ceph_namespace}" exec deploy/rook-ceph-tools -- ceph pg map "${pg_id}" --format json | jq --arg currentOsd "${stuck_osd_id}" 'del(.up[] | select(. == ($currentOsd | tonumber))).up[-1]')
        [[ -z "${new_primary_osd}" || "${new_primary_osd}" == "${stuck_osd_id}" ]] || kubectl -n "${rook_ceph_namespace}" exec deploy/rook-ceph-tools -- ceph osd primary-temp "${pg_id}" "${new_primary_osd}"
    done
}


function wait_until_pg_drained(){
    local rook_ceph_namespace="rook-ceph"
    local start_time
    local current_time
    local timeout=1800
    local diff

    start_time=$(date +"%s")
    while line=$(kubectl -n "${rook_ceph_namespace}" exec deploy/rook-ceph-tools -- ceph osd df --format json | jq -r --argjson pv_osd_id "${PV_OSD_ID}"  '.nodes[] | select((.id as $osd_id | $pv_osd_id | index($osd_id)) and .pgs>0) | (.id | tostring)  + "=" + (.pgs | tostring)') && echo "$line" | grep -q '.';  do
        current_time=$(date +"%s")
        diff=$((current_time-start_time))
        [[ $diff -gt $timeout ]] && error "Timeout waiting for PGs to drained from PV OSDs"
        for line_itr in $line; do
            id=$(echo "${line_itr}" | cut -d'=' -f1)
            pg=$(echo "${line_itr}" | cut -d'=' -f2)
            # If PGs gets stuck during drain , it need special handling
            # We invoke this handling function when all PGs are active clean
            is_ceph_pg_active_clean && handle_pgs_stuck_in_drain "${id}"
            echo "Waiting for '${pg}' PGs to move out of osd.'$id'"
        done
        sleep 10
        echo "-------------------"
    done
}

####
# Waits for all rook-ceph pods to be updated to latest version and checks ceph health in the end
####
function check_latest_ceph_health() {
    
    if validate_rook_ceph_version;
    then
        # If all pods are successfully updated, check ceph health
        info "Rook ceph already upgraded to latest version"
        return 0
    fi

    osd_deploy_orig=$(kubectl -n rook-ceph get cephcluster rook-ceph -o json | jq '.spec.mon.count')
    
    local rook_cluster_namespace=rook-ceph
    local try=0
    local maxtry=100
    info "Waiting for rook-ceph to be updated to latest version.. ${try}/${maxtry}"
    while ((try != maxtry));
    do
        osd_count=$(kubectl -n rook-ceph get deploy -l app=rook-ceph-osd --no-headers 2>/dev/null | wc -l)
        if [[ "${osd_count}" -ge "${osd_deploy_orig}" ]];
        then
            if validate_rook_ceph_version;
            then
                info "Rook-Ceph updated to latest version"
                break
            fi
        fi
        try=$((try+1))
        cleanup_crashing_osd
        info "Waiting for rook-ceph to be updated to latest version.. ${try}/${maxtry}"
        sleep 30
    done

    osd_deployments=$(kubectl -n rook-ceph get deploy -l app=rook-ceph-osd -o name)
    for deploy in ${osd_deployments};
    do
        kubectl -n rook-ceph rollout status "$deploy" --timeout=3m || cleanup_crashing_osd
    done

    kubectl -n rook-ceph delete pods -l app=rook-ceph-rgw
    gateway_deploy=$(kubectl -n rook-ceph get deploy -l app=rook-ceph-rgw -o name)
    for deploy in ${gateway_deploy};
    do
        kubectl -n rook-ceph rollout status "$deploy" --timeout=3m || true
    done

    if validate_rook_ceph_version;
    then
        # If all pods are successfully updated, check ceph health
        ceph_health_exit_on_error
        info "Rook ceph upgrade successful"
    else
        # As all pods could not be updated to latest version, exit
        kubectl -n $rook_cluster_namespace get deployments -l rook_cluster=$rook_cluster_namespace -o jsonpath='{range .items[*]}{.metadata.name}{"  \treq/upd/avl: "}{.spec.replicas}{"/"}{.status.updatedReplicas}{"/"}{.status.readyReplicas}{"  \trook-version="}{.metadata.labels.rook-version}{"\n"}{end}'
        error "Rook-Ceph could not be updated to latest version"
    fi
}

function main() {
    if [[ "$(is_rook_ceph_installed)" == "false" ]]; then
        warn "Skipping rook-ceph-post-1.8.9.sh because rook-ceph is not installed"
        return
    fi
    migrate_pgs_from_pv_to_raw_osd_if_possible
    check_latest_ceph_health
}

main
exit 0
