#!/bin/bash

# Copyright UiPath 2021
#
# =================
# LICENSE AGREEMENT
# -----------------
#   Use of paid UiPath products and services is subject to the licensing agreement
#   executed between you and UiPath. Unless otherwise indicated by UiPath, use of free
#   UiPath products is subject to the associated licensing agreement available here:
#   https://www.uipath.com/legal/trust-and-security/legal-terms (or successor website).
#   You must not use this file separately from the product it is a part of or is associated with.

set -e
#GLOBALS
SCRIPT_DIR=$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)
if [[ -z "${SCRIPT_DIR}" ]]; then
  error_with_code "10001"
fi
MODULES_DIR=$(cd "${SCRIPT_DIR}/Modules" && pwd)
if [[ -z "${MODULES_DIR}" ]]; then
  error_with_code "10002"
fi
PLAYBOOKS_DIR=$(cd "${SCRIPT_DIR}/Playbooks" && pwd)
if [[ -z "${PLAYBOOKS_DIR}" ]]; then
    error_with_code "10002"
fi
GOTEMPLATES_DIR=$(cd "${SCRIPT_DIR}/Tools/gotemplates" && pwd)
if [[ -z "${GOTEMPLATES_DIR}" ]]; then
  error_with_code "10002"
fi

readonly SCRIPT_DIR
readonly MODULES_DIR

export ANSIBLE_STDOUT_CALLBACK=yaml
export PATH=$PATH:/var/lib/rancher/rke2/bin:${SCRIPT_DIR}/bin:/usr/local/bin
#shellcheck disable=SC1090,SC1091
source "${MODULES_DIR}/utils.sh"
#shellcheck disable=SC1090,SC1091
source "${MODULES_DIR}/install_packages/prerequisites-rhel.sh"
#shellcheck disable=SC1090,SC1091
source "${MODULES_DIR}/rhel-utils.sh"
#shellcheck disable=SC1090,SC1091
source "${MODULES_DIR}/offline-utils.sh"
#shellcheck disable=SC1090,SC1091
source "${MODULES_DIR}/argocd_utils.sh"
#shellcheck disable=SC1090,SC1091
source "${MODULES_DIR}"/cluster_utils.sh
#shellcheck disable=SC1090,SC1091,SC2034
source "${MODULES_DIR}/hadr_utils.sh"


# ACTIONS
OPS=0
GENERATE_INVENTORY=1
GENERATE_CLUSTER_CONFIG=1
GENERATE_BACKUP_CONFIG=1
CONFIGURE_BACKUP=0
PACKAGES_INSTALL=0
GENERATE_VARIABLES=1
TMP_CEPH_BACKUP_PATH=${TMP_CEPH_BACKUP_PATH:-""}

UPGRADE_CNI="true"

function set_defaults() {
    INSTALLER_VERSION=$(jq -r '.version' 2>/dev/null < "${SCRIPT_DIR}/defaults.json") || error_with_code "14014"
    TARGET_BASE_PATH="/opt/UiPathAutomationSuite/${INSTALLER_VERSION}"
    TARGET_INSTALLER_BASE_PATH="${TARGET_BASE_PATH}/installer"
#    MERGED_INPUT_FILE="/opt/UiPathAutomationSuite/UiPath_Installer/merged_input.json"
    # DEFAULTS
    LOCAL_TEMP_PATH="/var/tmp/uipathctl_${INSTALLER_VERSION}"
    LOGFILE_PATH="${LOCAL_TEMP_PATH}/_install-uipath.log"
    OPS_LOGFILE_PATH="${LOCAL_TEMP_PATH}/_ops-manager.log"
    [[ -z "${INVENTORY}" ]] && INVENTORY="${LOCAL_TEMP_PATH}/inventory.ini"
    [[ -z "${VARIABLES_FILE}" ]] && VARIABLES_FILE="${LOCAL_TEMP_PATH}/variables.yaml"
    GENERATED_RESTORE_FILE="${LOCAL_TEMP_PATH}/restore.json"
    [[ -z "${BACKUP_CONFIG_FILE}" ]] && BACKUP_CONFIG_FILE="${LOCAL_TEMP_PATH}/backup.json"
    [[ -z "${CLUSTER_CONFIG_FILE}" ]] && CLUSTER_CONFIG_FILE="${LOCAL_TEMP_PATH}/cluster_config.json"

    KUBECONFIG_PATH="/etc/rancher/rke2/rke2.yaml"
    TARGET_BUNDLE_TEMP_PATH="${TARGET_BASE_PATH}/.ansible"

    [[ -z "${KUBECONFIG_PATH}" ]] && KUBECONFIG_PATH="/etc/rancher/rke2/rke2.yaml"

    # EXPORTS
    export ANSIBLE_LOG_PATH="${LOGFILE_PATH}"
}

function display_usage() {
  echo
  echo "***************************************************************************************"
  echo
  echo "uipathctl.sh runs cluster-wide operations for Automation Suite using Ansible."
  echo
  echo "Usage:"
  echo "  $COMMAND_NAME [command]"
  echo "  $COMMAND_NAME [flags]"
  echo
  echo "Available Commands:"
  echo "  upgrade                                           Upgrade cluster"
  echo "  configure-backup                                  Configure backup on all nodes"
  echo "  disable-backup                                    Disable backup on all nodes"
  echo "  restore                                           Restore cluster from existing backup on a fresh set of nodes"
  echo "  install-prerequisites                             Install required prerequisites (python3, ansible, sshpass, zip, unzip). If offline, --offline-prerequisites-bundle is also required"
  echo "  multisite-primary-setup                           Convert an existing cluster into multisite primary cluster"
  echo "  get-alternate-kubeconfig                          Creates a user in the current cluster utilized by the other cluster in setting up multisite environment by pulling necessary information needed for the setup"
  echo
  echo "Granular Upgrade steps:"
  echo "  --steps <all|bundles|infra|fabric|services>       Steps to run during upgrade operations. Defaults to all."
  echo "                                                    all:       executes all steps"
  echo "                                                    bundles:   copies required offline bundles to all nodes"
  echo "                                                    infra:     executes infra installation on server and agent nodes"
  echo "                                                    fabric:    executes fabric installation on a server"
  echo "                                                    services:  executes services installation on a server"
  echo
  echo "Parameters:"
  echo "  --install-type <online|offline>                   Type of installation. Required for upgrade and restore."
  echo "  --cluster-config                                  Path to your cluster_config.json file. A config file will be generated if not specified."
  echo "  --kubeconfig                                      Path to a Kubeconfig file which allows connecting to the cluster. Defaults to /etc/rancher/rke2/rke2.yaml"
  echo "  --inventory                                       Path to a valid Ansible inventory describing the cluster. Inventory will be generated if not specified."
  echo "  --backup-endpoint                                 The NFS server which will be used for generating a default backup configuration."
  echo "  --backup-mount                                    The mount path of the NFS server where backups will be stored."
  echo "  --backup-name                                     An identifier for the backup of this cluster in particular."
  echo "  --backup-config                                   Path to a valid backup configuration JSON."
  echo "  --restore-config                                  Path to a valid restore configuration JSON."
  echo "  --ansible-username                                The username which should be used by Ansible to connect to cluster nodes. Defaults to the current user."
  echo "  --ansible-private-key                             The ssh private key which provides connectivity to cluster nodes. A set of passwords will be prompted if not specified."
  echo "  --ansible-ask-password                            Ask Ansible to request a password set for SSH connections and elevation."
  echo "  --ansible-config                                  Specify the path to a custom ansible.cfg to be used."
  echo "  --offline-bundle                                  Path to the sf.tar.gz file. Required for offline upgrades."
  echo "  --offline-infra-bundle                            Path to the sf-infra.tar.gz file. Required for offline upgrades."
  echo "  --offline-prerequisites-bundle                    Path to the ansible.tar.gz. Required for installing ansible in offline environments."
  echo "  --disable-cni-upgrade                             Do not perform CNI upgrade."
  echo
  echo "Flags:"
  echo "  -h|--help                                         Display help"
  echo "  -d|--debug                                        Run in debug mode"
  echo
  echo "***************************************************************************************"
  echo
}

function contains() {
  if [[ $1 =~ (^|[[:space:]])$2($|[[:space:]]) ]]; then
    echo "0"
  else
    echo "1"
  fi
}

function parse_long_args() {
    set +e
    while (("$#")); do
        case "$1" in
        -d | --debug)
            info "Running in debug"
            DEBUG=1
            shift
            set -x
            ;;

        -h | --help)
            display_usage
            shift
            exit 0
            ;;

        --kubeconfig)
            if [ -n "$2" ] && [ "${2:0:1}" != "-" ]; then
                ABSOLUTEPATH=$(readlink -f "$2")
                [[ -f "${ABSOLUTEPATH}" ]] || error_with_code "10006" "$2"
                KUBECONFIG_PATH="$2"
                shift 2
            else
                display_usage
                error_with_code "10005" "$1"
                shift 2
            fi
            ;;

        upgrade)
            UPGRADE=1
            OPS=$((OPS=OPS+1))
            shift
            ;;

        configure-backup)
            CONFIGURE_BACKUP=1
            OPS=$((OPS=OPS+1))
            shift
            ;;

        restore)
            RESTORE=1
            OPS=$((OPS=OPS+1))
            shift
            ;;

        disable-backup)
            CONFIGURE_BACKUP=1
            BACKUP_ARGS="--disable-backup"
            OPS=$((OPS=OPS+1))
            shift
            ;;

        install-prerequisites)
            PACKAGES_INSTALL=1
            OPS=$((OPS=OPS+1))
            shift
            ;;

        multisite-primary-setup)
            HADR_PRIMARY_SETUP=1
            OPS=$((OPS=OPS+1))
            shift
            ;;

        get-alternate-kubeconfig)
            RETRIEVE_ALTERNATE_KUBECONFIG=1
            OPS=$((OPS=OPS+1))
            shift
            ;;

        --inventory)
            if [ -n "$2" ] && [ "${2:0:1}" != "-" ]; then
                local ABSOLUTEPATH
                ABSOLUTEPATH=$(readlink -f "$2")
                [[ -f "${ABSOLUTEPATH}" ]] || error_with_code "10032" "$2"
                INVENTORY=$ABSOLUTEPATH
                GENERATE_INVENTORY=0
                shift 2
            else
                display_usage
                error_with_code "10005" "$1"
                shift 2
            fi
            ;;

        --install-type)
            if [ -n "$2" ] && [ "${2:0:1}" != "-" ]; then
                INSTALL_TYPE="$2"
                [[ "${INSTALL_TYPE}" != "online" && "${INSTALL_TYPE}" != "offline" ]] && display_usage && error_with_code "10028" "${INSTALL_TYPE}"
                shift 2
            else
                display_usage
                error_with_code "10005" "$1"
                shift 2
            fi
            ;;

        --steps)
            if [ -n "$2" ] && [ "${2:0:1}" != "-" ]; then
                STEPS="$2"
                result=$(contains "all infra fabric services bundles cni" "${STEPS}")
                [[ "${result}" == "1" ]] && display_usage && error "Unsupported value for --steps: ${STEPS}."
                [[ "${STEPS}" == "all" ]] && unset STEPS
                shift 2
            else
                display_usage
                error_with_code "10005" "$1"
                shift 2
            fi
            ;;

        --backup-endpoint)
            if [ -n "$2" ] && [ "${2:0:1}" != "-" ]; then
                BACKUP_ENDPOINT="$2"
                shift 2
            else
                display_usage
                error_with_code "10005" "$1"
                shift 2
            fi
            ;;

        --backup-mount)
            if [ -n "$2" ] && [ "${2:0:1}" != "-" ]; then
                BACKUP_MOUNT="$2"
                shift 2
            else
                display_usage
                error_with_code "10005" "$1"
                shift 2
            fi
            ;;

        --backup-name)
            if [ -n "$2" ] && [ "${2:0:1}" != "-" ]; then
                BACKUP_NAME="$2"
                shift 2
            else
                display_usage
                error_with_code "10005" "$1"
                shift 2
            fi
            ;;

        --backup-config)
            if [ -n "$2" ] && [ "${2:0:1}" != "-" ]; then
                local ABSOLUTEPATH
                ABSOLUTEPATH=$(readlink -f "$2")
                [[ -f "${ABSOLUTEPATH}" ]] || error_with_code "10032" "$2"
                BACKUP_CONFIG_FILE=$ABSOLUTEPATH
                GENERATE_BACKUP_CONFIG=0
                shift 2
            fi
            ;;

        --cluster-config)
            if [ -n "$2" ] && [ "${2:0:1}" != "-" ]; then
                local ABSOLUTEPATH
                ABSOLUTEPATH=$(readlink -f "$2")
                [[ -f "${ABSOLUTEPATH}" ]] || error_with_code "10032" "$2"
                CLUSTER_CONFIG_FILE=$ABSOLUTEPATH
                GENERATE_CLUSTER_CONFIG=0
                shift 2
            else
                display_usage
                error_with_code "10005" "$1"
                shift 2
            fi
            ;;

        --restore-config)
            if [ -n "$2" ] && [ "${2:0:1}" != "-" ]; then
                local ABSOLUTEPATH
                ABSOLUTEPATH=$(readlink -f "$2")
                [[ -f "${ABSOLUTEPATH}" ]] || error_with_code "10032" "$2"
                RESTORE_CONFIG_FILE=$ABSOLUTEPATH
                GENERATE_CLUSTER_CONFIG=0
                shift 2
            else
                display_usage
                error_with_code "10005" "$1"
                shift 2
            fi
            ;;

        --ansible-username)
            if [ -n "$2" ] && [ "${2:0:1}" != "-" ]; then
                ANSIBLE_USERNAME="$2"
                shift 2
            else
                display_usage
                error_with_code "10005" "$1"
                shift 2
            fi
            ;;

        --ansible-ask-password)
            ANSIBLE_ASK_PASSWORD=1
            shift
            ;;

        --ansible-config)
            if [ -n "$2" ] && [ "${2:0:1}" != "-" ]; then
                local ABSOLUTEPATH
                ABSOLUTEPATH=$(readlink -f "$2")
                [[ -f "${ABSOLUTEPATH}" ]] || error_with_code "10032" "$2"
                export ANSIBLE_CONFIG=$ABSOLUTEPATH
                shift 2
            fi
            ;;
        --ansible-private-key)
            if [ -n "$2" ] && [ "${2:0:1}" != "-" ]; then
                local ABSOLUTEPATH
                ABSOLUTEPATH=$(readlink -f "$2")
                [[ -f "${ABSOLUTEPATH}" ]] || error_with_code "10032" "$2"
                ANSIBLE_PRIVATE_KEY=$ABSOLUTEPATH
                shift 2
            else
                display_usage
                error_with_code "10005" "$1"
                shift 2
            fi
            ;;

        --ansible-variables-file)
            if [ -n "$2" ] && [ "${2:0:1}" != "-" ]; then
                local ABSOLUTEPATH
                ABSOLUTEPATH=$(readlink -f "$2")
                [[ -f "${ABSOLUTEPATH}" ]] || error_with_code "10032" "$2"
                VARIABLES_FILE=$ABSOLUTEPATH
                GENERATE_VARIABLES=0
                shift 2
            else
                display_usage
                error_with_code "10005" "$1"
                shift 2
            fi
            ;;
        --offline-bundle)
            if [ -n "$2" ] && [ "${2:0:1}" != "-" ]; then
                local ABSOLUTEPATH
                ABSOLUTEPATH=$(readlink -f "$2")
                [[ -f "${ABSOLUTEPATH}" ]] || error_with_code "10032" "$2"
                OFFLINE_BUNDLE_PATH=$ABSOLUTEPATH
                shift 2
            else
                display_usage
                error_with_code "10005" "$1"
                shift 2
            fi
            ;;

        --offline-infra-bundle)
            if [ -n "$2" ] && [ "${2:0:1}" != "-" ]; then
                local ABSOLUTEPATH
                ABSOLUTEPATH=$(readlink -f "$2")
                [[ -f "${ABSOLUTEPATH}" ]] || error_with_code "10032" "$2"
                [[ "${ABSOLUTEPATH}" == "${TARGET_BASE_PATH}/_auto_sf-infra-only.tar.gz" ]] && error_with_code "10044" "$ABSOLUTEPATH"
                OFFLINE_INFRA_BUNDLE_PATH=$ABSOLUTEPATH
                shift 2
            else
                display_usage
                error_with_code "10005" "$1"
                shift 2
            fi
            ;;

        --offline-prerequisites-bundle)
            if [ -n "$2" ] && [ "${2:0:1}" != "-" ]; then
                local ABSOLUTEPATH
                ABSOLUTEPATH=$(readlink -f "$2")
                [[ -f "${ABSOLUTEPATH}" ]] || error_with_code "10032" "$2"
                OFFLINE_ANSIBLE_BUNDLE_PATH=$ABSOLUTEPATH
                shift 2
            else
                display_usage
                error_with_code "10005" "$1"
                shift 2
            fi
            ;;

        --accept-license-agreement)
            export LICENSE_AGREEMENT="accept"
            shift
            ;;

        --disable-cni-upgrade)
            UPGRADE_CNI="false"
            shift
            ;;

        -*|*) # unsupported flags
            display_usage
            error_with_code "10007" "$1"
            exit 1
            ;;
        esac
    done

    # ERRORS
    [[ "${OPS}" == "0" ]] && display_usage && error_with_code "14001"
    [[ "${CONFIGURE_BACKUP}" != "1" ]] && [[ -z ${INSTALL_TYPE} ]] && display_usage && error_with_code "14002"
    (( OPS > 1 )) && error_with_code "14003"
    [[ "${INSTALL_TYPE}" == "offline" ]] && [[ "${UPGRADE}" == "1" ]] && [[ -z ${STEPS} || "${STEPS}" == "bundles" ]] && [[ -z ${OFFLINE_INFRA_BUNDLE_PATH} || -z ${OFFLINE_BUNDLE_PATH} ]] && error_with_code "14006"
    [[ "${INSTALL_TYPE}" == "offline" ]] && [[ "${RESTORE}" == "1" ]] && [[ -z ${OFFLINE_INFRA_BUNDLE_PATH} ]] && error_with_code "14007"
    [[ "${INSTALL_TYPE}" == "online" ]] && [[ -n ${OFFLINE_INFRA_BUNDLE_PATH} || -n ${OFFLINE_BUNDLE_PATH} ]] && error_with_code "14008"
    [[ "${CONFIGURE_BACKUP}" == "1" ]] && [[ -z ${BACKUP_CONFIG_FILE} ]] && [[ -z ${BACKUP_ENDPOINT} ]] && error_with_code "14009"
    [[ "${RESTORE}" == "1" ]] && [[ -z "${RESTORE_CONFIG_FILE}" ]] && error_with_code "14010"
    [[ "${RESTORE}" == "1" ]] && [[ "${GENERATE_INVENTORY}" == "1" ]] && error_with_code "14011"
    [[ -n ${BACKUP_ENDPOINT} ]] && [[ -z ${BACKUP_MOUNT} ]] && error_with_code "14012" "--backup-mount"
    [[ -n ${BACKUP_ENDPOINT} ]] && [[ -z ${BACKUP_NAME} ]] && error_with_code "14012" "--backup-name"
    [[ -n ${ANSIBLE_PRIVATE_KEY} ]] && [[ -n ${ANSIBLE_ASK_PASSWORD} ]] && error_with_code "14013" "--ansible-private-key" "--ansible-ask-password"
    [[ "${PACKAGES_INSTALL}" == "1" ]] && [[ -z "${OFFLINE_ANSIBLE_BUNDLE_PATH}" ]] && [[ "${INSTALL_TYPE}" == "offline" ]] && error_with_code "14016"

    # WARNINGS
    if [[ "${PACKAGES_INSTALL}" == "0" ]]; then
        [[ -z ${INVENTORY} ]] && warn_with_code "54001" "${LOCAL_TEMP_PATH}/inventory.ini"
        [[ ${GENERATE_CLUSTER_CONFIG} == "1" ]] && warn_with_code "54002"
        [[ -z ${ANSIBLE_USERNAME} ]] && warn_with_code "54004" "${USER}" && ANSIBLE_USERNAME=${USER}
    fi
    [[ "${CONFIGURE_BACKUP}" == "1" ]] && [[ ${GENERATE_BACKUP_CONFIG} == "0" ]] && [[ -n ${BACKUP_ENDPOINT} ]] && warn_with_code "54003" "${BACKUP_CONFIG_FILE}" && unset BACKUP_ENDPOINT
    [[ "${CONFIGURE_BACKUP}" == "1" ]] && [[ -n ${BACKUP_ENDPOINT} ]] && warn_with_code "54005" "${BACKUP_CONFIG_FILE}"
    [[ "${UPGRADE}" == "0" ]] && [[ -n "${STEPS}" ]] && warn_with_code "54007" && unset STEPS
    [[ "${HADR_PRIMARY_SETUP}" == "0" ]] && [[ -n "${STEPS}" ]] && warn_with_code "54007" && unset STEPS
    if [[ -n "${TMP_CEPH_BACKUP_PATH}" ]]; then
        [[ -e "${TMP_CEPH_BACKUP_PATH}" && -d "${TMP_CEPH_BACKUP_PATH}" ]] || error "Invalid folder: '${TMP_CEPH_BACKUP_PATH}' provided for environment variable: 'TMP_CEPH_BACKUP_PATH'"
    fi
    set -e
}

function get_agents_of_type() {
    agentType=$1
    local template
    template="${GOTEMPLATES_DIR}/${agentType}.gotemplate"
    [[ -z ${template} ]] && error "Cannot find ${template}."
    echo "$template"
}

function generate_inventory() {
    rm -f "${INVENTORY}"
    # shellcheck disable=SC2016
    cat >> "${INVENTORY}" <<EOF
[FIRST_SERVER]
$(kubectl get nodes -o go-template-file="$(get_agents_of_type server)" | head -n 1 | awk '{ print "\x27"$1"\x27" }')

[SECONDARY_SERVERS]
$(kubectl get nodes -o go-template-file="$(get_agents_of_type server)" | tail -n +2 | awk '{ print "\x27"$1"\x27" }')

[AGENTS]
$(kubectl get nodes -o go-template-file="$(get_agents_of_type agent)" | awk '{ print "\x27"$1"\x27" }')

[TASKMINING]
$(kubectl get nodes -o go-template-file="$(get_agents_of_type tm)" | awk '{ print "\x27"$1"\x27" }')

[GPU]
$(kubectl get nodes -o go-template-file="$(get_agents_of_type gpu)" | awk '{ print "\x27"$1"\x27" }')

[all:vars]
ansible_connection=ssh
ansible_timeout=10
ansible_user=${ANSIBLE_USERNAME}
EOF
}

function execute_playbooks() {
    [[ ${CONFIGURE_BACKUP} == "1" ]] && execute_ansible_playbook_command "${PLAYBOOKS_DIR}/backup.yaml"
    [[ ${UPGRADE} == "1" ]] && export UPGRADE_SILENT_MODE="false" && cluster_backup
    if [[ ${UPGRADE} == "1" ]]; then
        [[ "${INSTALL_TYPE}" == "online" ]] && execute_ansible_playbook_command "${PLAYBOOKS_DIR}/online.yaml"
        [[ "${INSTALL_TYPE}" == "offline" ]] && execute_ansible_playbook_command "${PLAYBOOKS_DIR}/offline.yaml"
    fi
    if [[ ${RESTORE} == "1" ]]; then
        [[ "${INSTALL_TYPE}" == "online" ]] && execute_ansible_playbook_command "${PLAYBOOKS_DIR}/restore-online.yaml"
        [[ "${INSTALL_TYPE}" == "offline" ]] && execute_ansible_playbook_command "${PLAYBOOKS_DIR}/restore-offline.yaml"
    fi
    if [[ ${HADR_PRIMARY_SETUP} == "1" ]]; then
        [[ "${INSTALL_TYPE}" == "online" ]] && execute_ansible_playbook_command "${PLAYBOOKS_DIR}/standalone-to-primary-online.yaml"
        [[ "${INSTALL_TYPE}" == "offline" ]] && execute_ansible_playbook_command "${PLAYBOOKS_DIR}/standalone-to-primary-offline.yaml"
    fi
}

function archive_installer() {
    INSTALLER="${LOCAL_TEMP_PATH}/installer.zip"
    info "Trying to delete ${INSTALLER} if it already exists..."
    rm -f "${INSTALLER}"
    info "Zipping installer folder to ${INSTALLER}..."
    cd "${SCRIPT_DIR}/"
    zip -r "${INSTALLER}" "." -2 -x '*.git*' 1>/dev/null
    [[ -f "${INSTALLER}" ]] || error_with_code "10032"
}

function generate_variables_file() {
    rm -f "${VARIABLES_FILE}"
    touch "${VARIABLES_FILE}"
    [[ -n ${INSTALLER} ]] && echo "installer_path: ${INSTALLER}" >> "${VARIABLES_FILE}"
    [[ -n ${TARGET_INSTALLER_BASE_PATH} ]] && echo "target_installer_base_path: ${TARGET_INSTALLER_BASE_PATH}" >> "${VARIABLES_FILE}"
    cat >> "${VARIABLES_FILE}" <<EOF
install_type: ${INSTALL_TYPE}
target_bundle_base_path: ${TARGET_BASE_PATH}
tmp_ceph_backup_path: ${TMP_CEPH_BACKUP_PATH}
EOF

    if [[ -n ${OFFLINE_INFRA_BUNDLE_PATH} ]]; then
        echo "infra_bundle_filename: $(basename "${OFFLINE_INFRA_BUNDLE_PATH}")" >> "${VARIABLES_FILE}"
        echo "infra_bundle_basepath: $(dirname "${OFFLINE_INFRA_BUNDLE_PATH}")" >> "${VARIABLES_FILE}"
    fi

    if [[ -n $RESTORE_CONFIG_FILE ]]; then
        echo "restore_config_filename: $(basename "${RESTORE_CONFIG_FILE}")" >> "${VARIABLES_FILE}"
        echo "restore_config_basepath: $(dirname "${RESTORE_CONFIG_FILE}")" >> "${VARIABLES_FILE}"
    fi

    if [[ ${CONFIGURE_BACKUP} == "1" ]]; then
        echo "backup_config_filename: $(basename "${BACKUP_CONFIG_FILE}")" >> "${VARIABLES_FILE}"
        echo "backup_config_basepath: $(dirname "${BACKUP_CONFIG_FILE}")" >> "${VARIABLES_FILE}"
    fi

    echo "cluster_config_filename: $(basename "${CLUSTER_CONFIG_FILE}")" >> "${VARIABLES_FILE}"
    echo "cluster_config_basepath: $(dirname "${CLUSTER_CONFIG_FILE}")" >> "${VARIABLES_FILE}"
}

function generate_restore_config() {
    rm -f "${GENERATED_RESTORE_FILE}"
    touch "${GENERATED_RESTORE_FILE}"
    cat >> "${GENERATED_RESTORE_FILE}" << EOF
{
    "fixed_rke_address": "$(jq -r '.fixed_rke_address' < "${CLUSTER_CONFIG_FILE}")",
    "gpu_support": "$(jq -r '.gpu_support' < "${CLUSTER_CONFIG_FILE}")",
    "fqdn": "$(jq -r '.fqdn' < "${CLUSTER_CONFIG_FILE}")",
    "rke_token": "$(jq -r '.rke_token' < "${CLUSTER_CONFIG_FILE}")",
    "restore": {
        "etcdRestorePath": "$(jq -r '.backup.etcdBackupPath' < "${BACKUP_CONFIG_FILE}")",
        "nfs": {
        "endpoint": "$(jq -r '.backup.nfs.endpoint' < "${BACKUP_CONFIG_FILE}")",
        "mountpath": "$(jq -r '.backup.nfs.mountpath' < "${BACKUP_CONFIG_FILE}")"
        }
    },
    "infra": $(jq -r '.infra' < "${CLUSTER_CONFIG_FILE}")
}
EOF
}

function generate_backup_config() {
    rm -f "${BACKUP_CONFIG_FILE}"
    touch "${BACKUP_CONFIG_FILE}"
    cat >> "${BACKUP_CONFIG_FILE}" <<EOF
{
  "backup": {
    "etcdBackupPath": "${BACKUP_NAME}",
    "nfs": {
      "endpoint": "${BACKUP_ENDPOINT}",
      "mountpath": "${BACKUP_MOUNT}"
    }
  },
  "backup_interval": "15"
}
EOF
}

function prepare_environment() {
    mkdir -p "$LOCAL_TEMP_PATH"
    mkdir -p "$TARGET_BUNDLE_TEMP_PATH"
}

function validate_prereqs() {
    ansible-playbook --version &>/dev/null || error_with_code "14018" "ansible-playbook"
    if [[ "${ANSIBLE_ASK_PASSWORD}" == "1" ]]; then
        sshpass -V &>/dev/null || error_with_code "14019"
    fi
    zip --version &>/dev/null || error_with_code "14018" "zip"
    unzip &>/dev/null || error_with_code "14018" "unzip"
}

function install_prereqs() {
    [[ -z "${OFFLINE_ANSIBLE_BUNDLE_PATH}" ]] && install_online_ctl_prereqs
    [[ -n "${OFFLINE_ANSIBLE_BUNDLE_PATH}" ]] && install_offline_ansible "${OFFLINE_ANSIBLE_BUNDLE_PATH}" "${LOCAL_TEMP_PATH}"
    exit 0
}

function write_operation_log() {
    cat >> "${OPS_LOGFILE_PATH}" <<EOF
=================================
TIMESTAMP: $(date +%s)

INVENTORY: ${INVENTORY}
$(cat "${INVENTORY}")

VARIABLES FILE: ${VARIABLES_FILE}
$(cat "${VARIABLES_FILE}")

ANSIBLE ARGUMENTS:
EOF
    echo "${ANSIBLE_ARGS[@]}" "--extra-vars \"" "${EXTRA_VARS[@]}" "\"" >> "${OPS_LOGFILE_PATH}"
    info "Logfile for debugging: ${OPS_LOGFILE_PATH}"
}

function execute_ansible_playbook_command() {
    declare -a ANSIBLE_ARGS
    declare -a EXTRA_VARS
    local playbook
    playbook=$1
    local verbosity_level="-v"
    [[ "${DEBUG}" == 1 ]] && verbosity_level="-vvv"
    ANSIBLE_ARGS+=("\"${playbook}\"")
    ANSIBLE_ARGS+=("-i \"${INVENTORY}\"")
    [[ -n ${STEPS} ]] && ANSIBLE_ARGS+=("--tags \"${STEPS}\"")
    [[ -n ${ANSIBLE_PRIVATE_KEY} ]] && ANSIBLE_ARGS+=("--private-key \"${ANSIBLE_PRIVATE_KEY}\"")
    [[ -n ${ANSIBLE_ASK_PASSWORD} ]] && ANSIBLE_ARGS+=(--ask-pass) && ANSIBLE_ARGS+=(--ask-become-pass)
    ANSIBLE_ARGS+=("${verbosity_level}")

    EXTRA_VARS+=("variables_file_path=${VARIABLES_FILE}")
    EXTRA_VARS+=("local_temp_path=${LOCAL_TEMP_PATH}")
    EXTRA_VARS+=("target_bundle_temp_path=${TARGET_BUNDLE_TEMP_PATH}")
    EXTRA_VARS+=("upgrade_cni=${UPGRADE_CNI}")
    [[ -n "${BACKUP_ARGS}" ]] && EXTRA_VARS+=("backup_args=${BACKUP_ARGS}")
    write_operation_log
    eval ansible-playbook "${ANSIBLE_ARGS[*]}" --extra-vars "\"${EXTRA_VARS[*]}\"" || error "Execution failed. Logs located at ${LOGFILE_PATH}; Debug information located at ${OPS_LOGFILE_PATH}."
}

function check_eula() {
    [[ -z "${LICENSE_AGREEMENT}" ]] || EULA="${LICENSE_AGREEMENT}"
    if [[ "${EULA}" != "accept" ]]; then
      echo "Use of paid UiPath products and services is subject to the licensing agreement executed between you and UiPath."
      echo "Unless otherwise indicated by UiPath, use of free UiPath products is subject to the associated licensing agreement available here: https://www.uipath.com/legal/trust-and-security/legal-terms (or successor website)."
      echo "By setting the environment variable LICENSE_AGREEMENT=accept or by using the --accept-license-agreement argument, you agree to the applicable licensing agreement."
      exit 0
    fi
}

function main() {
    parse_long_args "$@"
    check_eula
    set_defaults
    [[ "${PACKAGES_INSTALL}" == "1" ]] && install_prereqs
    validate_prereqs
    prepare_environment
    archive_installer
    export KUBECONFIG="${KUBECONFIG_PATH}"
    [[ "${UPGRADE}" == "1" || "${HADR_PRIMARY_SETUP}" == "1" ]] && start_incoming_connections_istio && wait_for_istio_pods
    [[ "${RETRIEVE_ALTERNATE_KUBECONFIG}" == "1" ]] && retrieve_alternate_cluster_kubeconfig
    [[ "${GENERATE_CLUSTER_CONFIG}" == "1" ]] && eval "${SCRIPT_DIR}/configureUiPathAS.sh config get -o ${CLUSTER_CONFIG_FILE} --kubeconfig ${KUBECONFIG}"
    [[ "${CONFIGURE_BACKUP}" == "1" && "${GENERATE_BACKUP_CONFIG}" == "1" ]] && generate_backup_config && generate_restore_config
    [[ "${GENERATE_INVENTORY}" == "1" ]] && generate_inventory
    [[ "${GENERATE_VARIABLES}" == "1" ]] && generate_variables_file
    if [[ -n "${OFFLINE_BUNDLE_PATH}" ]] && [[ "${UPGRADE}" == "1" || "${HADR_PRIMARY_SETUP}" == "1" ]] && [[ -z "${STEPS}" || "${STEPS}" == "bundles" ]]; then
        eval "${SCRIPT_DIR}/configureUiPathAS.sh registry upload --offline-bundle ${OFFLINE_BUNDLE_PATH} --offline-tmp-folder /uipath/tmp" || error_with_code "14017"
    fi
    [[ "${RETRIEVE_ALTERNATE_KUBECONFIG:-0}" != "1" ]] && execute_playbooks
}

main "$@"
